#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2019 Purism SPC
# SPDX-License-Identifier: GPL-3.0+
# Author: David Boddie <david.boddie@puri.sm>

from evsupport import Tester
import time


class PhoshTester(Tester):

    default_delay = 0.5

    def __init__(self, device_file=None,
                 portrait=True, panel_open=False, locked=False,
                 kbd_open=False, favorites_open=False):

        super().__init__(device_file)

        self.portrait = portrait
        self.panel_open = panel_open
        self.locked = locked
        self.kbd_open = kbd_open
        self.favorites_open = favorites_open

    def unlock(self, passcode="123456"):

        self.tap(360, 720)

        if self.portrait:
            self.swipe((300, 950), (300, 700), 20, 0.01)
            time.sleep(1)

            keys = {
                "1": (225, 610), "2": (360, 610), "3": (490, 610),
                "4": (230, 770), "5": (360, 770), "6": (490, 770),
                "7": (230, 930), "8": (360, 930), "9": (490, 930),
                "0": (360, 1100)
            }

            # Enter the passcode
            for key in passcode:
                if key in keys:
                    self.tap(*keys[key])

            # Submit
            self.tap(360, 1300)

        self.locked = False

    def open_kbd(self):

        if self.kbd_open:
            return

        self.tap(700, 1400)
        self.kbd_open = True

    def close_kbd(self):

        if not self.kbd_open:
            return

        self.tap(700, 1100)
        self.kbd_open = False

    def open_panel(self):

        if self.panel_open:
            return

        if self.portrait:
            self.tap(360, 20)
        else:
            self.tap(700, 720)

        self.panel_open = True

    def close_panel(self):

        if not self.panel_open:
            return

        if self.portrait:
            self.tap(360, 20)
        else:
            self.tap(700, 720)

        self.panel_open = False

    def set_volume(self, x):

        """Sets the volume using the slider in the panel.
        Minimum and maximum values are [124, 645] (portrait),
                                       [466, 1024] (landscape).
        """

        if not self.panel_open:
            self.open_panel()
            time.sleep(self.default_delay)

        if self.portrait:
            self.tap(x, 100)
        else:
            self.tap(600, x)

    def set_brightness(self, x):

        """Sets the brightness using the slider in the panel.
        Minimum and maximum values are [124, 645] (portrait),
                                       [466, 1024] (landscape).
        """

        if not self.panel_open:
            self.open_panel()
            time.sleep(self.default_delay)

        if self.portrait:
            self.tap(x, 190)
        else:
            self.tap(510, x)

    def rotate(self):

        if not self.panel_open:
            self.open_panel()
            time.sleep(self.default_delay)

        if self.portrait:
            #self.tap(620, 300)
            self.swipe((580,300), (620,300), 10, 0.02)
        else:
            #self.tap(450, 950)
            self.swipe((430,1010), (430,970), 10, 0.02)

        self.portrait = not self.portrait
        self.panel_open = False

    def lock(self):

        if not self.panel_open:
            self.open_panel()
            time.sleep(self.default_delay)

        if self.portrait:
            self.tap(360, 475)
        else:
            self.tap(300, 720)

        self.panel_open = False

    def settings(self):

        if not self.panel_open:
            self.open_panel()
            time.sleep(self.default_delay)

        if self.portrait:
            self.tap(180, 475)
        else:
            self.tap(300, 840)

        self.panel_open = False

    def open_favorites(self):

        if self.favorites_open:
            return

        if self.portrait:
            self.tap(360, 1420)
        else:
            self.tap(20, 720)

        self.favorites_open = True

    def close_favorites(self):

        if not self.favorites_open:
            return

        if self.portrait:
            self.tap(360, 1420)
        else:
            self.tap(20, 720)

        self.favorites_open = False

    keys = {
        "a": (100, 1220), "b": (380, 1320), "c": (280, 1320), "d": (200, 1220),
        "e": (150, 1120), "f": (280, 1220), "g": (330, 1220), "h": (380, 1220),
        "i": (430, 1120), "j": (430, 1220), "k": (480, 1220), "l": (540, 1220),
        "m": (480, 1320), "n": (430, 1320), "o": (480, 1120), "p": (540, 1120),
        "q": (30, 1120), "r": (180, 1120), "s": (150, 1220), "t": (280, 1120),
        "u": (400, 1120), "v": (340, 1320), "w": (80, 1120), "x": (180, 1320),
        "y": (340, 1120), "z": (120, 1320), ",": (550, 1320), ".": (600, 1320),
        " ": (360, 1420),
        "caps": (40, 1320), "ctrl": (40, 1420), "escape": (700, 1320),
        "backspace": (700, 1120), "return": (700, 1220),
        "123": (710, 1430), "abc": (710, 1430),
        "-": (100, 1220), "=": (380, 1320), '"': (280, 1320), "*": (200, 1220),
        "3": (150, 1120), "^": (280, 1220), ":": (330, 1220), ";": (380, 1220),
        "8": (430, 1120), "(": (430, 1220), ")": (480, 1220), "~": (540, 1220),
        "!": (480, 1320), "?": (430, 1320), "9": (480, 1120), "0": (540, 1120),
        "1": (30, 1120), "4": (180, 1120), "@": (150, 1220), "5": (280, 1120),
        "7": (400, 1120), "+": (340, 1320), "2": (80, 1120), "'": (180, 1320),
        "6": (340, 1120), "/": (120, 1320), "\\": (550, 1320), "|": (600, 1320),
    }

    def press_key(self, key):

        self.tap(*self.keys[key])

    def enter_text(self, text):

        if not self.kbd_open:
            self.open_kbd()

        for char in text:
            self.tap(*self.keys[char])

    def close_window(self):

        # Assumes that there is a close button at the top-right corner of the
        # screen.
        if self.portrait:
            self.tap(700, 100)
        else:
            self.tap(640, 1420)

    def back(self):

        # Assumes that there is a back button at the top-left corner of the
        # screen.
        if self.portrait:
            self.tap(40, 100)
        else:
            self.tap(640, 40)

    def tap_top_left_icon(self):
        self.back()
